@@ -8,8 +8,7 @@ from textwrap import dedent
 from typing import TYPE_CHECKING
 
 import pytest
-from griffe.docstrings.dataclasses import DocstringSectionExamples, DocstringSectionKind
-from griffe.tests import temporary_visited_module
+from griffe import DocstringSectionExamples, DocstringSectionKind, temporary_visited_module
 
 from mkdocstrings_handlers.python.handler import CollectionError, PythonHandler, get_handler
 
@@ -120,10 +119,11 @@ def test_expand_globs_without_changing_directory() -> None:
         (True, {"extension.py:SomeExtension": {"option": "value"}}),
         (True, {"path/to/extension.py": {"option": "value"}}),
         (True, {"path/to/extension.py:SomeExtension": {"option": "value"}}),
-        (False, "/absolute/path/to/extension.py"),
-        (False, "/absolute/path/to/extension.py:SomeExtension"),
-        (False, {"/absolute/path/to/extension.py": {"option": "value"}}),
-        (False, {"/absolute/path/to/extension.py:SomeExtension": {"option": "value"}}),
+        # True because OS path normalization.
+        (True, "/absolute/path/to/extension.py"),
+        (True, "/absolute/path/to/extension.py:SomeExtension"),
+        (True, {"/absolute/path/to/extension.py": {"option": "value"}}),
+        (True, {"/absolute/path/to/extension.py:SomeExtension": {"option": "value"}}),
         (False, "dot.notation.path.to.extension"),
         (False, "dot.notation.path.to.pyextension"),
         (False, {"dot.notation.path.to.extension": {"option": "value"}}),
